@@ -1,12 +1,11 @@
 //go:build go1.18 && !go1.19
-// +build go1.18,!go1.19
 
 package quic
 
 import (
 	"crypto/cipher"
 
-	"github.com/marten-seemann/qtls-go1-18"
+	"github.com/quic-go/qtls-go1-18"
 )
 
 type (
